@@ -75,6 +75,7 @@ class AdminTool:
                 self.help[k[5:]] = getattr(self, k)
         self.tracker_home = ''
         self.db = None
+        self.db_uncommitted = False
 
     def get_class(self, classname):
         '''Get the class - raise an exception if it doesn't exist.
@@ -286,26 +287,31 @@ Command help:
 
         Look in the following places, where the later rules take precedence:
 
-         1. <prefix>/share/roundup/templates/*
+         1. <roundup.admin.__file__>/../../share/roundup/templates/*
+            this is where they will be if we installed an egg via easy_install
+         2. <prefix>/share/roundup/templates/*
             this should be the standard place to find them when Roundup is
             installed
-         2. <roundup.admin.__file__>/../templates/*
+         3. <roundup.admin.__file__>/../templates/*
             this will be used if Roundup's run in the distro (aka. source)
             directory
-         3. <current working dir>/*
+         4. <current working dir>/*
             this is for when someone unpacks a 3rd-party template
-         4. <current working dir>
+         5. <current working dir>
             this is for someone who "cd"s to the 3rd-party template dir
         '''
         # OK, try <prefix>/share/roundup/templates
+        #     and <egg-directory>/share/roundup/templates
         # -- this module (roundup.admin) will be installed in something
         # like:
-        #    /usr/lib/python2.2/site-packages/roundup/admin.py  (5 dirs up)
-        #    c:\python22\lib\site-packages\roundup\admin.py     (4 dirs up)
-        # we're interested in where the "lib" directory is - ie. the /usr/
-        # part
+        #    /usr/lib/python2.5/site-packages/roundup/admin.py  (5 dirs up)
+        #    c:\python25\lib\site-packages\roundup\admin.py     (4 dirs up)
+        #    /usr/lib/python2.5/site-packages/roundup-1.3.3-py2.5-egg/roundup/admin.py
+        #    (2 dirs up)
+        #
+        # we're interested in where the directory containing "share" is
         templates = {}
-        for N in 4, 5:
+        for N in 2, 4, 5:
             path = __file__
             # move up N elements in the path
             for i in range(N):
@@ -642,6 +648,7 @@ Erase it? Y/N: """))
             except (TypeError, IndexError, ValueError), message:
                 import traceback; traceback.print_exc()
                 raise UsageError, message
+        self.db_uncommitted = True
         return 0
 
     def do_find(self, args):
@@ -749,7 +756,7 @@ Erase it? Y/N: """))
             keys.sort()
             for key in keys:
                 value = cl.get(nodeid, key)
-                print _('%(key)s: %(value)r')%locals()
+                print _('%(key)s: %(value)s')%locals()
 
     def do_create(self, args):
         ""'''Usage: create classname property=value ...
@@ -813,6 +820,7 @@ Erase it? Y/N: """))
             print apply(cl.create, (), props)
         except (TypeError, IndexError, ValueError), message:
             raise UsageError, message
+        self.db_uncommitted = True
         return 0
 
     def do_list(self, args):
@@ -995,6 +1003,7 @@ Erase it? Y/N: """))
         they are successful.
         '''
         self.db.commit()
+        self.db_uncommitted = False
         return 0
 
     def do_rollback(self, args):
@@ -1007,6 +1016,7 @@ Erase it? Y/N: """))
         immediately after would make no changes to the database.
         '''
         self.db.rollback()
+        self.db_uncommitted = False
         return 0
 
     def do_retire(self, args):
@@ -1030,6 +1040,7 @@ Erase it? Y/N: """))
                 raise UsageError, _('no such class "%(classname)s"')%locals()
             except IndexError:
                 raise UsageError, _('no such %(classname)s node "%(nodeid)s"')%locals()
+        self.db_uncommitted = True
         return 0
 
     def do_restore(self, args):
@@ -1052,6 +1063,7 @@ Erase it? Y/N: """))
                 raise UsageError, _('no such class "%(classname)s"')%locals()
             except IndexError:
                 raise UsageError, _('no such %(classname)s node "%(nodeid)s"')%locals()
+        self.db_uncommitted = True
         return 0
 
     def do_export(self, args, export_files=True):
@@ -1216,6 +1228,7 @@ Erase it? Y/N: """))
             print 'setting', classname, maxid+1
             self.db.setid(classname, str(maxid+1))
 
+        self.db_uncommitted = True
         return 0
 
     def do_pack(self, args):
@@ -1254,6 +1267,7 @@ Erase it? Y/N: """))
         elif m['date']:
             pack_before = date.Date(value)
         self.db.pack(pack_before)
+        self.db_uncommitted = True
         return 0
 
     def do_reindex(self, args, desre=re.compile('([A-Za-z]+)([0-9]+)')):
@@ -1424,7 +1438,7 @@ Erase it? Y/N: """))
             self.run_command(args)
 
         # exit.. check for transactions
-        if self.db and self.db.transactions:
+        if self.db and self.db_uncommitted:
             commit = raw_input(_('There are unsaved changes. Commit them (y/N)? '))
             if commit and commit[0].lower() == 'y':
                 self.db.commit()
