@@ -14,7 +14,7 @@
 # FOR A PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS"
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
-# 
+#
 #$Id$
 '''This module exports file storage for roundup backends.
 Files are stored into a directory hierarchy.
@@ -36,22 +36,26 @@ def files_in_dir(dir):
     return num_files
 
 class FileStorage:
-    """Store files in some directory structure"""    
+    """Store files in some directory structure"""
+
+    def __init__(self, umask):
+        self.umask = umask
+
     def subdirFilename(self, classname, nodeid, property=None):
         """Determine what the filename and subdir for nodeid + classname is."""
         if property:
             name = '%s%s.%s'%(classname, nodeid, property)
         else:
-            # roundupdb.FileClass never specified the property name, so don't 
+            # roundupdb.FileClass never specified the property name, so don't
             # include it
             name = '%s%s'%(classname, nodeid)
-        
+
         # have a separate subdir for every thousand messages
         subdir = str(int(nodeid) / 1000)
         return os.path.join(subdir, name)
-    
+
     def filename(self, classname, nodeid, property=None, create=0):
-        '''Determine what the filename for the given node and optionally 
+        '''Determine what the filename for the given node and optionally
         property is.
 
         Try a variety of different filenames - the file could be in the
@@ -101,6 +105,10 @@ class FileStorage:
             # save off the rename action
             self.transactions.append((self.doStoreFile, (classname, nodeid,
                 property)))
+        # always set umask before writing to make sure we have the proper one
+        # in multi-tracker (i.e. multi-umask) or modpython scenarios
+        # the umask may have changed since last we set it.
+        os.umask(self.umask)
         open(name, 'wb').write(content)
 
     def getfile(self, classname, nodeid, property):
@@ -125,7 +133,7 @@ class FileStorage:
         '''Store the file as part of a transaction commit.
         '''
         # determine the name of the file to write to
-        name = self.filename(classname, nodeid, property)
+        name = self.filename(classname, nodeid, property, 1)
 
         # the file is currently ".tmp" - move it to its real name to commit
         if name.endswith('.tmp'):
@@ -155,4 +163,21 @@ class FileStorage:
             name += '.tmp'
         os.remove(name)
 
+    def isStoreFile(self, classname, nodeid):
+        '''See if there is actually any FileStorage for this node.
+           Is there a better way than using self.filename?
+        '''
+        try:
+            fname = self.filename(classname, nodeid)
+            return True
+        except IOError:
+            return False
+
+    def destroy(self, classname, nodeid):
+        '''If there is actually FileStorage for this node
+           remove it from the filesystem
+        '''
+        if self.isStoreFile(classname, nodeid):
+            os.remove(self.filename(classname, nodeid))
+
 # vim: set filetype=python ts=4 sw=4 et si
