@@ -10,6 +10,7 @@ from MimeWriter import MimeWriter
 
 from roundup.rfc2822 import encode_header
 from roundup import __version__
+from roundup.date import get_timezone
 
 try:
     from email.Utils import formatdate
@@ -30,6 +31,11 @@ class Mailer:
         self.debug = os.environ.get('SENDMAILDEBUG', '') \
             or config["MAIL_DEBUG"]
 
+        # set timezone so that things like formatdate(localtime=True)
+        # use the configured timezone
+        os.environ['TZ'] = get_timezone(self.config.TIMEZONE).tzname(None)
+        time.tzset()
+
     def get_standard_message(self, to, subject, author=None):
         '''Form a standard email message from Roundup.
 
@@ -60,7 +66,7 @@ class Mailer:
         writer.addheader('Subject', encode_header(subject, charset))
         writer.addheader('To', ', '.join(to))
         writer.addheader('From', author)
-        writer.addheader('Date', formatdate())
+        writer.addheader('Date', formatdate(localtime=True))
 
         # Add a unique Roundup header to help filtering
         writer.addheader('X-Roundup-Name', encode_header(tracker_name,
@@ -190,8 +196,8 @@ class SMTPConnection(smtplib.SMTP):
     ''' Open an SMTP connection to the mailhost specified in the config
     '''
     def __init__(self, config):
-
-        smtplib.SMTP.__init__(self, config.MAILHOST)
+        smtplib.SMTP.__init__(self, config.MAILHOST, port=config['MAIL_PORT'],
+                              local_hostname=config['MAIL_LOCAL_HOSTNAME'])
 
         # start the TLS if requested
         if config["MAIL_TLS"]:
