@@ -24,6 +24,7 @@ __docformat__ = 'restructuredtext'
 
 import re, os, smtplib, socket, time, random
 import cStringIO, base64, quopri, mimetypes
+import os.path
 
 from rfc2822 import encode_header
 
@@ -141,10 +142,10 @@ class IssueClass:
     #
     # Note that this list also includes properties
     # defined in the classic template:
-    # assignedto, topic, priority, status.
+    # assignedto, keyword, priority, status.
     (
         ''"title", ''"messages", ''"files", ''"nosy", ''"superseder",
-        ''"assignedto", ''"topic", ''"priority", ''"status",
+        ''"assignedto", ''"keyword", ''"priority", ''"status",
         # following properties are common for all hyperdb classes
         # they are listed here to keep things in one place
         ''"actor", ''"activity", ''"creator", ''"creation",
@@ -282,18 +283,17 @@ class IssueClass:
         # figure author information
         if msgid:
             authid = messages.get(msgid, 'author')
-            authname = users.get(authid, 'realname')
-            if not authname:
-                authname = users.get(authid, 'username', '')
-            authaddr = users.get(authid, 'address', '')
         else:
-            # "system message"
-            authid = None
-            authname = 'admin'
-            authaddr = self.db.config.ADMIN_EMAIL
+            authid = self.db.getuid()
+        authname = users.get(authid, 'realname')
+        if not authname:
+            authname = users.get(authid, 'username', '')
+        authaddr = users.get(authid, 'address', '')
 
-        if authaddr:
+        if authaddr and self.db.config.MAIL_ADD_AUTHOREMAIL:
             authaddr = " <%s>" % straddr( ('',authaddr) )
+        elif authaddr:
+            authaddr = ""
 
         # make the message body
         m = ['']
@@ -303,21 +303,37 @@ class IssueClass:
             m.append(self.email_signature(nodeid, msgid))
 
         # add author information
-        if authid:
-            if len(self.get(nodeid,'messages')) == 1:
+        if authid and self.db.config.MAIL_ADD_AUTHORINFO:
+            if msgid and len(self.get(nodeid, 'messages')) == 1:
                 m.append(_("New submission from %(authname)s%(authaddr)s:")
                     % locals())
-            else:
+            elif msgid:
                 m.append(_("%(authname)s%(authaddr)s added the comment:")
                     % locals())
-        else:
-            m.append(_("System message:"))
-        m.append('')
+            else:
+                m.append(_("Change by %(authname)s%(authaddr)s:") % locals())
+            m.append('')
 
         # add the content
         if msgid is not None:
             m.append(messages.get(msgid, 'content', ''))
 
+        # get the files for this message
+        message_files = []
+        if msgid :
+            for fileid in messages.get(msgid, 'files') :
+                # check the attachment size
+                filename = self.db.filename('file', fileid, None)
+                filesize = os.path.getsize(filename)
+                if filesize <= self.db.config.NOSY_MAX_ATTACHMENT_SIZE:
+                    message_files.append(fileid)
+                else:
+                    base = self.db.config.TRACKER_WEB
+                    link = "".join((base, files.classname, fileid))
+                    filename = files.get(fileid, 'name')
+                    m.append(_("File '%(filename)s' not attached - "
+                        "you can download it from %(link)s.") % locals())
+
         # add the change note
         if note:
             m.append(note)
@@ -336,12 +352,6 @@ class IssueClass:
         quopri.encode(content, content_encoded, 0)
         content_encoded = content_encoded.getvalue()
 
-        # get the files for this message
-        if msgid is None:
-            message_files = None
-        else:
-            message_files = messages.get(msgid, 'files')
-
         # make sure the To line is always the same (for testing mostly)
         sendto.sort()
 
