@@ -17,11 +17,12 @@
 #
 # $Id$
 
-import unittest, os, shutil, errno, imp, sys, time, pprint, sets
+import unittest, os, shutil, errno, imp, sys, time, pprint, sets, base64, os.path
 
 from roundup.hyperdb import String, Password, Link, Multilink, Date, \
     Interval, DatabaseError, Boolean, Number, Node
-from roundup import date, password, init, instance, configuration
+from roundup.mailer import Mailer
+from roundup import date, password, init, instance, configuration, support
 
 from mocknull import MockNull
 
@@ -54,7 +55,8 @@ def setupTracker(dirname, backend="anydbm"):
     except OSError, error:
         if error.errno not in (errno.ENOENT, errno.ESRCH): raise
     # create the instance
-    init.install(dirname, 'templates/classic')
+    init.install(dirname, os.path.join(os.path.dirname(__file__), '..',
+        'templates/classic'))
     init.write_select_db(dirname, backend)
     config.save(os.path.join(dirname, 'config.ini'))
     tracker = instance.open(dirname)
@@ -69,8 +71,8 @@ def setupSchema(db, create, module):
     priority = module.Class(db, "priority", name=String(), order=String())
     priority.setkey("name")
     user = module.Class(db, "user", username=String(), password=Password(),
-        assignable=Boolean(), age=Number(), roles=String(),
-        supervisor=Link('user'))
+        assignable=Boolean(), age=Number(), roles=String(), address=String(),
+        supervisor=Link('user'),realname=String())
     user.setkey("username")
     file = module.FileClass(db, "file", name=String(), type=String(),
         comment=String(indexme="yes"), fooz=Password())
@@ -82,14 +84,18 @@ def setupSchema(db, create, module):
     stuff = module.Class(db, "stuff", stuff=String())
     session = module.Class(db, 'session', title=String())
     msg = module.FileClass(db, "msg", date=Date(),
-                           author=Link("user", do_journal='no'))
+                           author=Link("user", do_journal='no'),
+                           files=Multilink('file'), inreplyto=String(),
+                           messageid=String(),
+                           recipients=Multilink("user", do_journal='no')
+                           )
     session.disableJournalling()
     db.post_init()
     if create:
         user.create(username="admin", roles='Admin',
             password=password.Password('sekrit'))
         user.create(username="fred", roles='User',
-            password=password.Password('sekrit'))
+            password=password.Password('sekrit'), address='fred@example.com')
         status.create(name="unread")
         status.create(name="in-progress")
         status.create(name="testing")
@@ -253,6 +259,42 @@ class DBTest(MyTestCase):
             m = self.db.issue.get(nid, "nosy"); m.sort()
             self.assertEqual(l, m)
 
+            # verify that when we pass None to an Multilink it sets
+            # it to an empty list
+            self.db.issue.set(nid, nosy=None)
+            if commit: self.db.commit()
+            self.assertEqual(self.db.issue.get(nid, "nosy"), [])
+
+    def testMultilinkChangeIterable(self):
+        for commit in (0,1):
+            # invalid nosy value assertion
+            self.assertRaises(IndexError, self.db.issue.create, title='spam',
+                nosy=['foo%s'%commit])
+            # invalid type for nosy create
+            self.assertRaises(TypeError, self.db.issue.create, title='spam',
+                nosy=1)
+            u1 = self.db.user.create(username='foo%s'%commit)
+            u2 = self.db.user.create(username='bar%s'%commit)
+            # try a couple of the built-in iterable types to make
+            # sure that we accept them and handle them properly
+            # try a set as input for the multilink
+            nid = self.db.issue.create(title="spam", nosy=set(u1))
+            if commit: self.db.commit()
+            self.assertEqual(self.db.issue.get(nid, "nosy"), [u1])
+            self.assertRaises(TypeError, self.db.issue.set, nid,
+                nosy='invalid type')
+            # test with a tuple
+            self.db.issue.set(nid, nosy=tuple())
+            if commit: self.db.commit()
+            self.assertEqual(self.db.issue.get(nid, "nosy"), [])
+            # make sure we accept a frozen set
+            self.db.issue.set(nid, nosy=frozenset([u1,u2]))
+            if commit: self.db.commit()
+            l = [u1,u2]; l.sort()
+            m = self.db.issue.get(nid, "nosy"); m.sort()
+            self.assertEqual(l, m)
+       
+
 # XXX one day, maybe...
 #    def testMultilinkOrdering(self):
 #        for i in range(10):
@@ -277,6 +319,15 @@ class DBTest(MyTestCase):
             if commit: self.db.commit()
             self.assertNotEqual(a, b)
             self.assertNotEqual(b, date.Date('1970-1-1.00:00:00'))
+            # The 1970 date will fail for metakit -- it is used
+            # internally for storing NULL. The others would, too
+            # because metakit tries to convert date.timestamp to an int
+            # for storing and fails with an overflow.
+            for d in [date.Date (x) for x in '2038', '1970', '0033', '9999']:
+                self.db.issue.set(nid, deadline=d)
+                if commit: self.db.commit()
+                c = self.db.issue.get(nid, "deadline")
+                self.assertEqual(c, d)
 
     def testDateUnset(self):
         for commit in (0,1):
@@ -498,6 +549,15 @@ class DBTest(MyTestCase):
         name2 = self.db.user.get('1', 'username')
         self.assertEqual(name1, name2)
 
+    def testDestroyBlob(self):
+        # destroy an uncommitted blob
+        f1 = self.db.file.create(content='hello', type="text/plain")
+        self.db.commit()
+        fn = self.db.filename('file', f1)
+        self.db.file.destroy(f1)
+        self.db.commit()
+        self.assertEqual(os.path.exists(fn), False)
+
     def testDestroyNoJournalling(self):
         self.innerTestDestroy(klass=self.db.session)
 
@@ -870,6 +930,47 @@ class DBTest(MyTestCase):
         self.assertEquals(self.db.indexer.search(['flebble'], self.db.issue),
             {'1': {}})
 
+    def testIndexingOnImport(self):
+        # import a message
+        msgcontent = 'Glrk'
+        msgid = self.db.msg.import_list(['content', 'files', 'recipients'],
+                                        [repr(msgcontent), '[]', '[]'])
+        msg_filename = self.db.filename(self.db.msg.classname, msgid,
+                                        create=1)
+        support.ensureParentsExist(msg_filename)
+        msg_file = open(msg_filename, 'w')
+        msg_file.write(msgcontent)
+        msg_file.close()
+
+        # import a file
+        filecontent = 'Brrk'
+        fileid = self.db.file.import_list(['content'], [repr(filecontent)])
+        file_filename = self.db.filename(self.db.file.classname, fileid,
+                                         create=1)
+        support.ensureParentsExist(file_filename)
+        file_file = open(file_filename, 'w')
+        file_file.write(filecontent)
+        file_file.close()
+
+        # import an issue
+        title = 'Bzzt'
+        nodeid = self.db.issue.import_list(['title', 'messages', 'files',
+            'spam', 'nosy', 'superseder'], [repr(title), repr([msgid]),
+            repr([fileid]), '[]', '[]', '[]'])
+        self.db.commit()
+
+        # Content of title attribute is indexed
+        self.assertEquals(self.db.indexer.search([title], self.db.issue),
+            {str(nodeid):{}})
+        # Content of message is indexed
+        self.assertEquals(self.db.indexer.search([msgcontent], self.db.issue),
+            {str(nodeid):{'messages':[str(msgid)]}})
+        # Content of file is indexed
+        self.assertEquals(self.db.indexer.search([filecontent], self.db.issue),
+            {str(nodeid):{'files':[str(fileid)]}})
+
+
+
     #
     # searching tests follow
     #
@@ -1150,8 +1251,8 @@ class DBTest(MyTestCase):
 
     def testFilteringMultilinkSort(self):
         # 1: []                 Reverse:  1: []
-        # 2: []                           2: []              
-        # 3: ['admin','fred']             3: ['fred','admin']       
+        # 2: []                           2: []
+        # 3: ['admin','fred']             3: ['fred','admin']
         # 4: ['admin','bleep','fred']     4: ['fred','bleep','admin']
         # Note the sort order for the multilink doen't change when
         # reversing the sort direction due to the re-sorting of the
@@ -1442,7 +1543,7 @@ class DBTest(MyTestCase):
             ['1', '2', '3', '4', '5', '8', '6', '7'])
         ae(filt(None, {}, [('+','messages.author'), ('+','messages')]),
             ['6', '7', '8', '5', '4', '3', '1', '2'])
-        # The following will sort by 
+        # The following will sort by
         # author.supervisor.username and then by
         # author.username
         # I've resited the tempation to implement recursive orderprop
@@ -1475,15 +1576,33 @@ class DBTest(MyTestCase):
     def testImportExport(self):
         # use the filtering setup to create a bunch of items
         ae, filt = self.filteringSetup()
+        # Get some stuff into the journal for testing import/export of
+        # journal data:
+        self.db.user.set('4', password = password.Password('xyzzy'))
+        self.db.user.set('4', age = 3)
+        self.db.user.set('4', assignable = True)
+        self.db.issue.set('1', title = 'i1', status = '3')
+        self.db.issue.set('1', deadline = date.Date('2007'))
+        self.db.issue.set('1', foo = date.Interval('1:20'))
+        p = self.db.priority.create(name = 'some_prio_without_order')
+        self.db.commit()
+        self.db.user.set('4', password = password.Password('123xyzzy'))
+        self.db.user.set('4', assignable = False)
+        self.db.priority.set(p, order = '4711')
+        self.db.commit()
+
         self.db.user.retire('3')
         self.db.issue.retire('2')
 
         # grab snapshot of the current database
         orig = {}
+        origj = {}
         for cn,klass in self.db.classes.items():
             cl = orig[cn] = {}
+            jn = origj[cn] = {}
             for id in klass.list():
                 it = cl[id] = {}
+                jn[id] = self.db.getjournal(cn, id)
                 for name in klass.getprops().keys():
                     it[name] = klass.get(id, name)
 
@@ -1522,11 +1641,13 @@ class DBTest(MyTestCase):
                     maxid = max(maxid, id)
                 self.db.setid(cn, str(maxid+1))
                 klass.import_journals(journals[cn])
+            # This is needed, otherwise journals won't be there for anydbm
+            self.db.commit()
         finally:
             shutil.rmtree('_test_export')
 
         # compare with snapshot of the database
-        for cn, items in orig.items():
+        for cn, items in orig.iteritems():
             klass = self.db.classes[cn]
             propdefs = klass.getprops(1)
             # ensure retired items are retired :)
@@ -1546,6 +1667,19 @@ class DBTest(MyTestCase):
                             raise
                         # don't get hung up on rounding errors
                         assert not l.__cmp__(value, int_seconds=1)
+        for jc, items in origj.iteritems():
+            for id, oj in items.iteritems():
+                rj = self.db.getjournal(jc, id)
+                # Both mysql and postgresql have some minor issues with
+                # rounded seconds on export/import, so we compare only
+                # the integer part.
+                for j in oj:
+                    j[1].second = float(int(j[1].second))
+                for j in rj:
+                    j[1].second = float(int(j[1].second))
+                oj.sort()
+                rj.sort()
+                ae(oj, rj)
 
         # make sure the retired items are actually imported
         ae(self.db.user.get('4', 'username'), 'blop')
@@ -1600,6 +1734,38 @@ class DBTest(MyTestCase):
             'nosy', 'priority', 'spam', 'status', 'superseder'])
         self.assertEqual(self.db.issue.list(), ['1'])
 
+    def testNosyMail(self) :
+        """Creates one issue with two attachments, one smaller and one larger
+           than the set max_attachment_size.
+        """
+        db = self.db
+        db.config.NOSY_MAX_ATTACHMENT_SIZE = 4096
+        res = dict(mail_to = None, mail_msg = None)
+        def dummy_snd(s, to, msg, res=res) :
+            res["mail_to"], res["mail_msg"] = to, msg
+        backup, Mailer.smtp_send = Mailer.smtp_send, dummy_snd
+        try :
+            f1 = db.file.create(name="test1.txt", content="x" * 20)
+            f2 = db.file.create(name="test2.txt", content="y" * 5000)
+            m  = db.msg.create(content="one two", author="admin",
+                files = [f1, f2])
+            i  = db.issue.create(title='spam', files = [f1, f2],
+                messages = [m], nosy = [db.user.lookup("fred")])
+
+            db.issue.nosymessage(i, m, {})
+            mail_msg = res["mail_msg"].getvalue()
+            self.assertEqual(res["mail_to"], ["fred@example.com"])
+            self.failUnless("From: admin" in mail_msg)
+            self.failUnless("Subject: [issue1] spam" in mail_msg)
+            self.failUnless("New submission from admin" in mail_msg)
+            self.failUnless("one two" in mail_msg)
+            self.failIf("File 'test1.txt' not attached" in mail_msg)
+            self.failUnless(base64.b64encode("xxx") in mail_msg)
+            self.failUnless("File 'test2.txt' not attached" in mail_msg)
+            self.failIf(base64.b64encode("yyy") in mail_msg)
+        finally :
+            Mailer.smtp_send = backup
+
 class ROTest(MyTestCase):
     def setUp(self):
         # remove previous test, ignore errors
@@ -1716,8 +1882,10 @@ class SchemaTest(MyTestCase):
         self.assertEqual(self.db.a.get(aid, 'name'), 'apple')
         self.assertEqual(self.db.a.get(aid, 'newstr'), None)
         self.assertEqual(self.db.a.get(aid, 'newint'), None)
-        self.assertEqual(self.db.a.get(aid, 'newnum'), None)
-        self.assertEqual(self.db.a.get(aid, 'newbool'), None)
+        # hack - metakit can't return None for missing values, and we're not
+        # really checking for that behavior here anyway
+        self.assert_(not self.db.a.get(aid, 'newnum'))
+        self.assert_(not self.db.a.get(aid, 'newbool'))
         self.assertEqual(self.db.a.get(aid, 'newdate'), None)
         self.assertEqual(self.db.b.get(aid, 'name'), 'bear')
         aid2 = self.db.a.create(name='aardvark', newstr='booz')
