@@ -104,11 +104,11 @@ class MailgwTestCase(unittest.TestCase, DiffHelper):
         self.chef_id = self.db.user.create(username='Chef',
             address='chef@bork.bork.bork', realname='Bork, Chef', roles='User')
         self.richard_id = self.db.user.create(username='richard',
-            address='richard@test', roles='User')
-        self.mary_id = self.db.user.create(username='mary', address='mary@test',
+            address='richard@test.test', roles='User')
+        self.mary_id = self.db.user.create(username='mary', address='mary@test.test',
             roles='User', realname='Contrary, Mary')
-        self.john_id = self.db.user.create(username='john', address='john@test',
-            alternate_addresses='jondoe@test\njohn.doe@test', roles='User',
+        self.john_id = self.db.user.create(username='john', address='john@test.test',
+            alternate_addresses='jondoe@test.test\njohn.doe@test.test', roles='User',
             realname='John Doe')
 
     def tearDown(self):
@@ -140,7 +140,7 @@ class MailgwTestCase(unittest.TestCase, DiffHelper):
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
@@ -154,7 +154,7 @@ Subject: [issue] Testing...
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
 
@@ -175,7 +175,7 @@ This is a test submission of a new issue.
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
 
@@ -189,7 +189,7 @@ This is a test submission of a new issue.
     def testAlternateAddress(self):
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: John Doe <john.doe@test>
+From: John Doe <john.doe@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
@@ -206,7 +206,7 @@ This is a test submission of a new issue.
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: Testing...
 
@@ -228,10 +228,10 @@ This is a test submission of a new issue.
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, mary@test, richard@test
+TO: chef@bork.bork.bork, mary@test.test, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, mary@test, richard@test
+To: chef@bork.bork.bork, mary@test.test, richard@test.test
 From: "Bork, Chef" <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -252,6 +252,88 @@ nosy: Chef, mary, richard
 status: unread
 title: Testing...
 
+_______________________________________________________________________
+Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+<http://tracker.example/cgi-bin/roundup.cgi/bugs/issue1>
+_______________________________________________________________________
+''')
+
+    def testNewIssueNoAuthorInfo(self):
+        self.db.config.MAIL_ADD_AUTHORINFO = 'no'
+        self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+To: issue_tracker@your.tracker.email.domain.example
+Message-Id: <dummy_test_message_id>
+Subject: [issue] Testing... [nosy=mary; assignedto=richard]
+
+This is a test submission of a new issue.
+''')
+        self.compareMessages(self._get_mail(),
+'''FROM: roundup-admin@your.tracker.email.domain.example
+TO: chef@bork.bork.bork, mary@test.test, richard@test.test
+Content-Type: text/plain; charset=utf-8
+Subject: [issue1] Testing...
+To: mary@test.test, richard@test.test
+From: "Bork, Chef" <issue_tracker@your.tracker.email.domain.example>
+Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+MIME-Version: 1.0
+Message-Id: <dummy_test_message_id>
+X-Roundup-Name: Roundup issue tracker
+X-Roundup-Loop: hello
+Content-Transfer-Encoding: quoted-printable
+
+This is a test submission of a new issue.
+
+----------
+assignedto: richard
+messages: 1
+nosy: Chef, mary, richard
+status: unread
+title: Testing...
+
+_______________________________________________________________________
+Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+<http://tracker.example/cgi-bin/roundup.cgi/bugs/issue1>
+_______________________________________________________________________
+''')
+
+    def testNewIssueNoAuthorEmail(self):
+        self.db.config.MAIL_ADD_AUTHOREMAIL = 'no'
+        self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+To: issue_tracker@your.tracker.email.domain.example
+Message-Id: <dummy_test_message_id>
+Subject: [issue] Testing... [nosy=mary; assignedto=richard]
+
+This is a test submission of a new issue.
+''')
+        self.compareMessages(self._get_mail(),
+'''FROM: roundup-admin@your.tracker.email.domain.example
+TO: chef@bork.bork.bork, mary@test.test, richard@test.test
+Content-Type: text/plain; charset=utf-8
+Subject: [issue1] Testing...
+To: mary@test.test, richard@test.test
+From: "Bork, Chef" <issue_tracker@your.tracker.email.domain.example>
+Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+MIME-Version: 1.0
+Message-Id: <dummy_test_message_id>
+X-Roundup-Name: Roundup issue tracker
+X-Roundup-Loop: hello
+Content-Transfer-Encoding: quoted-printable
+
+New submission from Bork, Chef:
+
+This is a test submission of a new issue.
+
+----------
+assignedto: richard
+messages: 1
+nosy: Chef, mary, richard
+status: unread
+title: Testing...
+
 _______________________________________________________________________
 Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 <http://tracker.example/cgi-bin/roundup.cgi/bugs/issue1>
@@ -271,7 +353,7 @@ _______________________________________________________________________
         self.doNewIssue()
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: mary <mary@test>
+From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -281,10 +363,10 @@ This is a second followup
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, richard@test
+TO: chef@bork.bork.bork, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, richard@test
+To: chef@bork.bork.bork, richard@test.test
 From: "Contrary, Mary" <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -295,7 +377,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-Contrary, Mary <mary@test> added the comment:
+Contrary, Mary <mary@test.test> added the comment:
 
 This is a second followup
 
@@ -313,7 +395,7 @@ _______________________________________________________________________
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -328,10 +410,10 @@ This is a followup
 
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, john@test, mary@test
+TO: chef@bork.bork.bork, john@test.test, mary@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, john@test, mary@test
+To: chef@bork.bork.bork, john@test.test, mary@test.test
 From: richard <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -342,7 +424,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-richard <richard@test> added the comment:
+richard <richard@test.test> added the comment:
 
 This is a followup
 
@@ -357,6 +439,49 @@ Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 _______________________________________________________________________
 ''')
 
+    def testPropertyChangeOnly(self):
+        self.doNewIssue()
+        oldvalues = self.db.getnode('issue', '1').copy()
+        oldvalues['assignedto'] = None
+        self.db.issue.set('1', assignedto=self.chef_id)
+        self.db.commit()
+        self.db.issue.nosymessage('1', None, oldvalues)
+
+        new_mail = ""
+        for line in self._get_mail().split("\n"):
+            if "Message-Id: " in line:
+                continue
+            if "Date: " in line:
+                continue
+            new_mail += line+"\n"
+
+        self.compareMessages(new_mail, """
+FROM: roundup-admin@your.tracker.email.domain.example
+TO: chef@bork.bork.bork, richard@test.test
+Content-Type: text/plain; charset=utf-8
+Subject: [issue1] Testing...
+To: chef@bork.bork.bork, richard@test.test
+From: "Bork, Chef" <issue_tracker@your.tracker.email.domain.example>
+X-Roundup-Name: Roundup issue tracker
+X-Roundup-Loop: hello
+X-Roundup-Version: 1.3.3
+MIME-Version: 1.0
+Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+Content-Transfer-Encoding: quoted-printable
+
+
+Change by Bork, Chef <chef@bork.bork.bork>:
+
+
+----------
+assignedto:  -> Chef
+
+_______________________________________________________________________
+Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
+<http://tracker.example/cgi-bin/roundup.cgi/bugs/issue1>
+_______________________________________________________________________
+""")
+
 
     #
     # FOLLOWUP TITLE MATCH
@@ -365,7 +490,7 @@ _______________________________________________________________________
         self.doNewIssue()
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 Subject: Re: Testing... [assignedto=mary; nosy=+john]
@@ -374,10 +499,10 @@ This is a followup
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, john@test, mary@test
+TO: chef@bork.bork.bork, john@test.test, mary@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, john@test, mary@test
+To: chef@bork.bork.bork, john@test.test, mary@test.test
 From: richard <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -388,7 +513,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-richard <richard@test> added the comment:
+richard <richard@test.test> added the comment:
 
 This is a followup
 
@@ -407,7 +532,7 @@ _______________________________________________________________________
         nodeid1 = self.doNewIssue()
         nodeid2 = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 Subject: Re: Testing... [assignedto=mary; nosy=+john]
@@ -417,13 +542,13 @@ This is a followup
 
         nodeid3 = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup2_dummy_id>
-Subject: Ang: Re: Testing... 
+Subject: Ang: Re: Testing...
 
 This is a followup
-''')        
+''')
         self.assertEqual(nodeid1, nodeid2)
         self.assertEqual(nodeid1, nodeid3)
 
@@ -432,7 +557,7 @@ This is a followup
         self.db.config.MAILGW_SUBJECT_CONTENT_MATCH = 'never'
         self.assertNotEqual(self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 Subject: Re: Testing...
@@ -447,7 +572,7 @@ This is a followup
         self.db.config.MAILGW_SUBJECT_CONTENT_MATCH = 'creation 00:00:01'
         self.assertNotEqual(self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 Subject: Re: Testing...
@@ -458,7 +583,7 @@ This is a followup
         self.db.config.MAILGW_SUBJECT_CONTENT_MATCH = 'creation +1d'
         self.assertEqual(self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 Subject: Re: Testing...
@@ -472,7 +597,7 @@ This is a followup
         self.db.config.ADD_AUTHOR_TO_NOSY = 'yes'
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: john@test
+From: john@test.test
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -483,10 +608,10 @@ This is a followup
 
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, richard@test
+TO: chef@bork.bork.bork, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, richard@test
+To: chef@bork.bork.bork, richard@test.test
 From: John Doe <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -497,7 +622,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-John Doe <john@test> added the comment:
+John Doe <john@test.test> added the comment:
 
 This is a followup
 
@@ -517,9 +642,9 @@ _______________________________________________________________________
         self.db.config.ADD_RECIPIENTS_TO_NOSY = 'yes'
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard@test
+From: richard@test.test
 To: issue_tracker@your.tracker.email.domain.example
-Cc: john@test
+Cc: john@test.test
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
 Subject: [issue1] Testing...
@@ -542,7 +667,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-richard <richard@test> added the comment:
+richard <richard@test.test> added the comment:
 
 This is a followup
 
@@ -563,7 +688,7 @@ _______________________________________________________________________
         self.db.config.MESSAGES_TO_AUTHOR = 'yes'
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: john@test
+From: john@test.test
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -573,10 +698,10 @@ This is a followup
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, john@test, richard@test
+TO: chef@bork.bork.bork, john@test.test, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, john@test, richard@test
+To: chef@bork.bork.bork, john@test.test, richard@test.test
 From: John Doe <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -587,7 +712,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-John Doe <john@test> added the comment:
+John Doe <john@test.test> added the comment:
 
 This is a followup
 
@@ -607,7 +732,7 @@ _______________________________________________________________________
         self.instance.config.ADD_AUTHOR_TO_NOSY = 'no'
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: john@test
+From: john@test.test
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -617,10 +742,10 @@ This is a followup
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, richard@test
+TO: chef@bork.bork.bork, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, richard@test
+To: chef@bork.bork.bork, richard@test.test
 From: John Doe <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -631,7 +756,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-John Doe <john@test> added the comment:
+John Doe <john@test.test> added the comment:
 
 This is a followup
 
@@ -650,9 +775,9 @@ _______________________________________________________________________
         self.instance.config.ADD_RECIPIENTS_TO_NOSY = 'no'
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard@test
+From: richard@test.test
 To: issue_tracker@your.tracker.email.domain.example
-Cc: john@test
+Cc: john@test.test
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
 Subject: [issue1] Testing...
@@ -675,7 +800,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-richard <richard@test> added the comment:
+richard <richard@test.test> added the comment:
 
 This is a followup
 
@@ -694,7 +819,7 @@ _______________________________________________________________________
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -714,7 +839,7 @@ Subject: [issue1] Testing... [assignedto=mary; nosy=+john]
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -734,7 +859,7 @@ Subject: [issue1] [assignedto=mary; nosy=+john]
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -766,7 +891,48 @@ Subject: [issue] Testing...
 
 This is a test submission of a new issue.
 '''
-        self.assertRaises(Unauthorized, self._handle_mail, message)
+        try:
+            self._handle_mail(message)
+        except Unauthorized, value:
+            body_diff = self.compareMessages(str(value), """
+You are not a registered user.
+
+Unknown address: fubar@bork.bork.bork
+""")
+
+            assert not body_diff, body_diff
+
+        else:
+            raise AssertionError, "Unathorized not raised when handling mail"
+
+        # Add Web Access role to anonymous, and try again to make sure
+        # we get a "please register at:" message this time.
+        p = [
+            self.db.security.getPermission('Create', 'user'),
+            self.db.security.getPermission('Web Access', None),
+        ]
+
+        self.db.security.role['anonymous'].permissions=p
+
+        try:
+            self._handle_mail(message)
+        except Unauthorized, value:
+            body_diff = self.compareMessages(str(value), """
+You are not a registered user. Please register at:
+
+http://tracker.example/cgi-bin/roundup.cgi/bugs/user?template=register
+
+...before sending mail to the tracker.
+
+Unknown address: fubar@bork.bork.bork
+""")
+
+            assert not body_diff, body_diff
+
+        else:
+            raise AssertionError, "Unathorized not raised when handling mail"
+
+        # Make sure list of users is the same as before.
         m = self.db.user.list()
         m.sort()
         self.assertEqual(l, m)
@@ -786,7 +952,7 @@ This is a test submission of a new issue.
         self.doNewIssue()
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: mary <mary@test>
+From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -800,10 +966,10 @@ A message with encoding (encoded oe =F6)
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, richard@test
+TO: chef@bork.bork.bork, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, richard@test
+To: chef@bork.bork.bork, richard@test.test
 From: "Contrary, Mary" <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -814,7 +980,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-Contrary, Mary <mary@test> added the comment:
+Contrary, Mary <mary@test.test> added the comment:
 
 A message with encoding (encoded oe =C3=B6)
 
@@ -832,7 +998,7 @@ _______________________________________________________________________
         self.doNewIssue()
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: mary <mary@test>
+From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -853,10 +1019,10 @@ A message with first part encoded (encoded oe =F6)
 ''')
         self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
-TO: chef@bork.bork.bork, richard@test
+TO: chef@bork.bork.bork, richard@test.test
 Content-Type: text/plain; charset=utf-8
 Subject: [issue1] Testing...
-To: chef@bork.bork.bork, richard@test
+To: chef@bork.bork.bork, richard@test.test
 From: "Contrary, Mary" <issue_tracker@your.tracker.email.domain.example>
 Reply-To: Roundup issue tracker <issue_tracker@your.tracker.email.domain.example>
 MIME-Version: 1.0
@@ -867,7 +1033,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-Contrary, Mary <mary@test> added the comment:
+Contrary, Mary <mary@test.test> added the comment:
 
 A message with first part encoded (encoded oe =C3=B6)
 
@@ -884,7 +1050,7 @@ _______________________________________________________________________
         self.doNewIssue()
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: mary <mary@test>
+From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -917,7 +1083,7 @@ xxxxxx
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -941,7 +1107,7 @@ X-Roundup-Loop: hello
 Content-Transfer-Encoding: quoted-printable
 
 
-richard <richard@test> added the comment:
+richard <richard@test.test> added the comment:
 
 This is a followup
 
@@ -976,7 +1142,7 @@ This is a followup
 
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -1029,7 +1195,7 @@ This is a followup
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: Re: Complete your registration to Roundup issue tracker
  -- key %s
@@ -1042,7 +1208,7 @@ This is a test confirmation of registration.
         self.db.keyword.create(name='Foo')
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -1055,9 +1221,9 @@ Subject: [keyword1] Testing... [name=Bar]
         nodeid = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
-Resent-From: mary <mary@test>
+Resent-From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
 
@@ -1076,7 +1242,7 @@ This is a test submission of a new issue.
 From: Chef <chef@bork.bork.bork>
 X-Roundup-Loop: hello
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: Re: [issue] Testing...
 
@@ -1090,7 +1256,7 @@ Hi, I've been mis-configured to loop messages back to myself.
 From: Chef <chef@bork.bork.bork>
 Precedence: bulk
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: Re: [issue] Testing...
 
@@ -1103,7 +1269,7 @@ Hi, I'm on holidays, and this is a dumb auto-responder.
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Message-Id: <dummy_test_message_id>
 Subject: Re: [issue] Out of office AutoReply: Back next week
 
@@ -1116,7 +1282,7 @@ Hi, I am back in the office next week
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1132,7 +1298,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [frobulated] testing
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1143,7 +1309,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [issue12345] testing
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1156,7 +1322,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [frobulated] testing
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1172,7 +1338,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: Re: [frobulated] testing
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1188,7 +1354,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [issue1234] testing
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1205,7 +1371,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [keyword1] Testing... [name=Bar]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1221,14 +1387,14 @@ Message-Id: <dummy_test_message_id>
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Subject: Testing... 
-Cc: richard@test
+Subject: Testing...
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
 '''
         self.assertRaises(MailUsageError, self._handle_mail, message)
-        
+
     def testClassStrictValid(self):
         self.instance.config.MAILGW_SUBJECT_PREFIX_PARSING = 'strict'
         self.instance.config.MAILGW_DEFAULT_CLASS = ''
@@ -1237,8 +1403,8 @@ Message-Id: <dummy_test_message_id>
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
-Subject: [issue] Testing... 
-Cc: richard@test
+Subject: [issue] Testing...
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1246,7 +1412,7 @@ Message-Id: <dummy_test_message_id>
 
         assert not os.path.exists(SENDMAILDEBUG)
         self.assertEqual(self.db.issue.get(nodeid, 'title'), 'Testing...')
-        
+
     #
     # TEST FOR INVALID COMMANDS HANDLING
     #
@@ -1257,7 +1423,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing [frobulated]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1270,7 +1436,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing [frobulated]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1286,7 +1452,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing [frobulated]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1302,7 +1468,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing [assignedto=mary]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1318,7 +1484,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing {assignedto=mary}
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1332,7 +1498,7 @@ Message-Id: <dummy_test_message_id>
         self.db.keyword.create(name='Foo')
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: richard <richard@test>
+From: richard <richard@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -1349,7 +1515,7 @@ Subject: {keyword1} Testing... {name=Bar}
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: testing [assignedto=mary]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1382,7 +1548,7 @@ In-Reply-To: <dummy_test_message_id2>
 Subject: Testing...
 
 Yet another message in the same thread/issue.
-''')        
+''')
 
         self.assertEqual(nodeid, nodeid2)
         self.assertEqual(nodeid, nodeid3)
@@ -1402,13 +1568,13 @@ Subject: hElp
 
     def testMaillistSubject(self):
         self.instance.config.MAILGW_SUBJECT_SUFFIX_DELIMITERS = '[]'
-        self.db.keyword.create(name='Foo')        
+        self.db.keyword.create(name='Foo')
         self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: [mailinglist-name] [keyword1] Testing.. [name=Bar]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1424,7 +1590,7 @@ Message-Id: <dummy_test_message_id>
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Subject: VeryStrangeRe: [keyword1] Testing.. [name=Bar]
-Cc: richard@test
+Cc: richard@test.test
 Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 
@@ -1437,7 +1603,7 @@ Message-Id: <dummy_test_message_id>
         nodeid1 = self.doNewIssue()
         nodeid2 = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
-From: mary <mary@test>
+From: mary <mary@test.test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
 In-Reply-To: <dummy_test_message_id>
@@ -1448,7 +1614,7 @@ This is a second followup
 
         assert nodeid1 == nodeid2
         self.assertEqual(self.db.issue.get(nodeid2, 'title'), "Testing...")
-        
+
 
 def test_suite():
     suite = unittest.TestSuite()
